@@ -111,7 +111,8 @@
 -import(ibrowse_lib, [
                       parse_url/1,
                       get_value/3,
-                      do_trace/2
+                      do_trace/2,
+                      log_msg/2
                      ]).
                       
 -record(state, {trace = false}).
@@ -300,8 +301,23 @@ send_req(Url, Headers, Method, Body) ->
 %%          {workaround, head_response_with_body}     |
 %%          {worker_process_options, list()} |
 %%          {return_raw_request, true}         |
-%%          {max_attempts, integer()}
+%%          {max_attempts, integer()}          |
+%%          {socks5_host, host()}              |
+%%          {socks5_port, integer()}           |
+%%          {socks5_user, binary()}            |
+%%          {socks5_password, binary()}
 %%
+%% ip4_address() = {0..255, 0..255, 0..255, 0..255}
+%% ip6_address() = 
+%%     {0..65535,
+%%      0..65535,
+%%      0..65535,
+%%      0..65535,
+%%      0..65535,
+%%      0..65535,
+%%      0..65535,
+%%      0..65535}
+%% host() = string() | ip4_address() | ip6_address()
 %% stream_to() = process() | {process(), once}
 %% process() = pid() | atom()
 %% username() = string()
@@ -481,12 +497,12 @@ do_send_req(Conn_Pid, Parsed_url, Headers, Method, Body, Options, Timeout) ->
                             _ ->
                                 process_info_not_available
                         end,
-            (catch lager:error("{ibrowse_http_client, send_req, ~1000.p} gen_server call timeout.~nProcess info: ~p~n",
-                               [[Conn_Pid, Parsed_url, Headers, Method, Body, Options, Timeout], P_info])),
+            log_msg("{ibrowse_http_client, send_req, ~1000.p} gen_server call timeout.~nProcess info: ~p~n",
+                    [[Conn_Pid, Parsed_url, Headers, Method, Body, Options, Timeout], P_info]),
             {error, req_timedout};
         {'EXIT', {normal, _}} = Ex_rsn ->
-            (catch lager:error("{ibrowse_http_client, send_req, ~1000.p} gen_server call got ~1000.p~n",
-                              [[Conn_Pid, Parsed_url, Headers, Method, Body, Options, Timeout], Ex_rsn])),
+            log_msg("{ibrowse_http_client, send_req, ~1000.p} gen_server call got ~1000.p~n",
+                    [[Conn_Pid, Parsed_url, Headers, Method, Body, Options, Timeout], Ex_rsn]),
             {error, req_timedout};
         {error, X} when X == connection_closed;
                         X == {send_failed, {error, enotconn}};
