@@ -262,17 +262,22 @@ handle_info({ssl_error, _Sock, Reason}, State) ->
     delayed_stop_timer(),
     {noreply, State_1};
 
-handle_info({req_timedout, From}, State) ->
-    case lists:keysearch(From, #request.from, queue:to_list(State#state.reqs)) of
+handle_info({req_timedout, From}, #state{reqs = Reqs} = State) ->
+    Reqs_list = queue:to_list(Reqs),
+    case lists:keysearch(From, #request.from, Reqs_list) of
         false ->
             {noreply, State};
         {value, #request{stream_to = StreamTo, req_id = ReqId}} ->
             catch StreamTo ! {ibrowse_async_response_timeout, ReqId},
             State_1 = State#state{proc_state = ?dead_proc_walking},
             shutting_down(State_1),
-            do_error_reply(State_1, req_timedout),
+            Reqs_1 = lists:filter(fun(#request{from = X_from}) ->
+                                          X_from /= From
+                                  end, Reqs_list),
+            State_2 = State_1#state{reqs = queue:from_list(Reqs_1)},
+            do_error_reply(State_2, req_timedout),
             delayed_stop_timer(),
-            {noreply, State_1}
+            {noreply, State_2}
     end;
 
 handle_info(timeout, State) ->
@@ -539,9 +544,9 @@ handle_sock_closed(#state{reply_buffer = Buf, reqs = Reqs, http_status_code = SC
                         true ->
                             {ok, Status_line, Raw_headers, Body, Raw_req};
                         false when Give_raw_req == false ->
-                            {ok, SC, Headers, Buf};
+                            {ok, SC, Headers, Body};
                         false ->
-                            {ok, SC, Headers, Buf, Raw_req}
+                            {ok, SC, Headers, Body, Raw_req}
                     end,
             State_1 = do_reply(State, From, StreamTo, ReqId, Resp_format, Reply),
             case Retry_state of
@@ -568,11 +573,12 @@ do_connect(Host, Port, Options, #state{is_ssl      = true,
     ssl:connect(Host, Port, get_sock_options(Host, Options, SSLOptions), Timeout);
 do_connect(Host, Port, Options, _State, Timeout) ->
     Socks5Host = get_value(socks5_host, Options, undefined),
+    Sock_options = get_sock_options(Host, Options, []),
     case Socks5Host of
       undefined ->
-        gen_tcp:connect(Host, Port, get_sock_options(Host, Options, []), Timeout);
+        gen_tcp:connect(Host, Port, Sock_options, Timeout);
       _ ->
-        catch ibrowse_socks5:connect(Host, Port, Options)
+        catch ibrowse_socks5:connect(Host, Port, Options, Sock_options, Timeout)
     end.
 
 get_sock_options(Host, Options, SSLOptions) ->
@@ -1173,6 +1179,7 @@ parse_response(Data, #state{reply_buffer = Acc, reqs = Reqs,
                                             http_status_code=StatCode};
                           false ->
                               State_0#state{recvd_headers=Headers_1, status=get_body,
+                                            status_line = Status_line,
                                             reply_buffer = <<>>,
                                             http_status_code=StatCode}
                       end,
@@ -1257,7 +1264,7 @@ parse_response(Data, #state{reply_buffer = Acc, reqs = Reqs,
                 undefined when HttpVsn =:= "HTTP/1.0";
                                ConnClose =:= "close" ->
                     send_async_headers(ReqId, StreamTo, Give_raw_headers, State_1),
-                    State_1#state{reply_buffer = Data_1};
+                    accumulate_response(Data_1, State_1);
                 undefined when StatCode =:= "303" ->
                     %% Some servers send 303 requests without a body.
                     %% RFC2616 says that they SHOULD, but they dont.
